@@ -1,7 +1,7 @@
 import io
 import time
 import timeit
-from typing import Callable, List, Optional, Union
+from typing import Callable, List, Literal, Optional, Union
 
 import dufte
 import matplotlib.animation as animation
@@ -74,10 +74,10 @@ class PerfplotData:
                 logx = False
             else:
                 log_n_range = np.log(self.n_range)
-                diff = log_n_range - np.linspace(
-                    log_n_range[0], log_n_range[-1], len(log_n_range)
-                )
-                logx = np.all(np.abs(diff) < 1.0e-5)
+                linlog = np.linspace(log_n_range[0], log_n_range[-1], len(log_n_range))
+                # don't consider first and last, they are equal anyway
+                rel_diff = (log_n_range - linlog)[1:-1] / log_n_range[1:-1]
+                logx = np.all(np.abs(rel_diff) <= 0.1)
 
         if logy == "auto":
             if relative_to is not None:
@@ -111,11 +111,10 @@ class PerfplotData:
                 scaled_timings = self.timings_s / self.timings_s[relative_to]
                 ylabel = f"Runtime\nrelative to {self.labels[relative_to]}"
 
-            # plt.title(ylabel)
-            dufte.ylabel(ylabel)
-
             for t, label in zip(scaled_timings, self.labels):
                 plotfun(self.n_range, t, label=label)
+
+            dufte.ylabel(ylabel)
         else:
             if relative_to is None:
                 flops = self.flop / self.timings_s
@@ -176,7 +175,13 @@ class Bench:
         self.setup = setup
         self.kernels = kernels
         self.equality_check = equality_check
-        self.n_range = n_range
+        # Make n_range a list of "scalar" np.arrays. This makes it possible for the
+        # setup function to override the value with
+        # n[...] = something_else.
+        # That is useful if you want to plot the graphs against something else than the
+        # input n.
+        assert len(np.asarray(n_range).shape) == 1
+        self.n_range = [np.array(item) for item in n_range]
         self.target_time_per_measurement = target_time_per_measurement
         self.max_time = max_time
         self.labels = labels
@@ -192,7 +197,10 @@ class Bench:
         if self.idx >= len(self.n_range):
             raise StopIteration
 
-        n = self.n_range[self.idx]
+        # Explicitly convert to Python int. One difference between a numpy.array(int)
+        # and a native Python int are in expressions like `n * [2.0]`.
+        n = int(self.n_range[self.idx])
+
         self.idx += 1
 
         data = None
@@ -319,11 +327,13 @@ def live(
     if labels is None:
         labels = [k.__name__ for k in kernels]
 
-    n_range = np.asarray(n_range)
+    # for data type of n_range, see above
+    assert len(np.asarray(n_range).shape) == 1
+    n_range = [np.array(item) for item in n_range]
 
     if logx == "auto":
         # Check if the x values are approximately equally spaced in log
-        if np.any(n_range <= 0):
+        if any(n <= 0 for n in n_range):
             logx = False
         else:
             log_n_range = np.log(n_range)
@@ -363,7 +373,7 @@ def live(
             lines.append(plotfun([], [], label=label)[0])
 
         ax.legend()
-        plt.ylabel("Runtime [s]")
+        dufte.ylabel("Runtime [s]")
         if xlabel:
             ax.set_xlabel(xlabel)
         xdata = []
@@ -423,7 +433,7 @@ def live(
 def bench(
     kernels: List[Callable],
     n_range: List[int],
-    setup: Optional[Callable] = None,
+    setup: Optional[Union[Callable, List[Callable]]] = None,
     flops: Optional[Callable] = None,
     labels: Optional[List[str]] = None,
     xlabel: Optional[str] = None,
@@ -469,6 +479,8 @@ def bench(
 
             for i in range(len(n_range)):
                 timings_s[i] = next(b)
+                # override n_rane in case it got overridden in next()
+                n_range = b.n_range
 
                 if show_progress:
                     assert task1 is not None
@@ -505,8 +517,8 @@ def show(
     *args,
     time_unit: str = "s",
     relative_to: Optional[int] = None,
-    logx: str = "auto",
-    logy: str = "auto",
+    logx: Union[bool, Literal["auto"]] = "auto",
+    logy: Union[bool, Literal["auto"]] = "auto",
     **kwargs,
 ):
     out = bench(*args, **kwargs)
@@ -523,8 +535,8 @@ def save(
     transparent=True,
     *args,
     time_unit: str = "s",
-    logx: str = "auto",
-    logy: str = "auto",
+    logx: Union[bool, Literal["auto"]] = "auto",
+    logy: Union[bool, Literal["auto"]] = "auto",
     relative_to: Optional[int] = None,
     **kwargs,
 ):
